@@ -250,7 +250,7 @@ GtkWidget* make_changer_buttons( void )
     int i;
     
     box = gtk_hbox_new(FALSE, 0);
-    changer_buttons = (GtkWidget**)g_new(GtkWidget, cd.nslots);
+    changer_buttons = g_new(GtkWidget *, cd.nslots);
     
     for(i=0; i < cd.nslots && i < 12; i++)
     {
@@ -567,6 +567,8 @@ void draw_status(void)
 
 gint slow_timer( gpointer *data )
 {
+    static char *lock_file = NULL;
+    
     tcd_post_init(&cd);
 
     /* see if we need to make a new menu */
@@ -585,7 +587,9 @@ gint slow_timer( gpointer *data )
     }
 
     /* is a cddb operation going on? */
-    cddb = g_file_test(gnome_util_home_file(".cddbstatus_lock"), G_FILE_TEST_EXISTS);
+    if (!lock_file)
+      lock_file = gnome_util_home_file(".cddbstatus_lock");
+    cddb = g_file_test(lock_file, G_FILE_TEST_EXISTS);
     
     draw_status(); 
     return 1;
@@ -793,6 +797,7 @@ void setup_time_display(GtkWidget *table)
 
     vol = gtk_adjustment_new (0.0, 0.0, 256.0, 0.1, 1.0, 1.0);
     volume = gtk_hscale_new(GTK_ADJUSTMENT(vol));
+    gtk_tooltips_set_tip(tooltips, volume, _("Volume"), "");
     gtk_range_set_update_policy( GTK_RANGE(volume), GTK_UPDATE_CONTINUOUS );
     gtk_scale_set_draw_value( GTK_SCALE(volume), FALSE );
     gtk_signal_connect( GTK_OBJECT(vol), "value_changed",
@@ -809,7 +814,8 @@ void setup_time_display(GtkWidget *table)
 			(GtkSignalFunc)status_click_event, NULL);
     gtk_widget_set_usize( status_area, 102, 60 );
 
-    gtk_widget_set_events (status_area, GDK_EXPOSURE_MASK
+    gtk_widget_set_events (status_area, gtk_widget_get_events(status_area)
+			   | GDK_EXPOSURE_MASK
 			   | GDK_LEAVE_NOTIFY_MASK
 			   | GDK_BUTTON_PRESS_MASK
 			   | GDK_POINTER_MOTION_MASK
@@ -965,20 +971,35 @@ void setup_keys()
     accel = gtk_accel_group_get_default();
 }    
 
+static char *CD_device = NULL;
+poptContext ctx;
+
+const struct poptOption gtcd_popt_options [] = {
+	{ "device", '\0', POPT_ARG_STRING, &CD_device, 0,
+	  N_("Filename of the CD device"),   N_("FILE") },
+	{ NULL, '\0', 0, NULL, 0 }
+};
+
+
+
 int main (int argc, char *argv[])
 {
     GtkWidget *table;
+    GdkColor black;
 
     bindtextdomain(PACKAGE, GNOMELOCALEDIR);
     textdomain(PACKAGE);
 
-    gnome_init("gtcd", VERSION, argc, argv);
+    gnome_init_with_popt_table("gtcd", VERSION, argc, argv, gtcd_popt_options, 0, &ctx);
 
     cd.play_method = NORMAL;
 
     load_prefs(&prefs);
-    cd.cdpath = prefs.cddev;
-	
+    if(CD_device)
+	cd.cdpath = CD_device;
+    else
+    	cd.cdpath = prefs.cddev;
+    		
     tcd_init_disc(&cd, (WarnFunc)create_warning);
 
     start_action();
@@ -1006,6 +1027,9 @@ int main (int argc, char *argv[])
     gnome_app_set_contents(GNOME_APP(window), main_box);
     adjust_status_size();
     gtk_widget_show_all(window);
+
+    gdk_color_black( gtk_widget_get_colormap(status_area), &black);
+    gdk_window_set_background( status_area->window, &black);
     
     signal(SIGUSR2, reload_info);
     
